@@ -28,6 +28,13 @@ import errno
 import os
 import re
 
+try:
+    from collections import OrderedDict
+except ImportError:
+    from dulwich._compat import OrderedDict
+
+from UserDict import DictMixin
+
 from dulwich.file import GitFile
 
 
@@ -36,7 +43,7 @@ class Config(object):
 
     def get(self, section, name):
         """Retrieve the contents of a configuration setting.
-        
+
         :param section: Tuple with section name and optional subsection namee
         :param subsection: Subsection name
         :return: Contents of the setting
@@ -65,7 +72,7 @@ class Config(object):
 
     def set(self, section, name, value):
         """Set a configuration value.
-        
+
         :param name: Name of the configuration value, including section
             and optional subsection
         :param: Value of the setting
@@ -73,13 +80,13 @@ class Config(object):
         raise NotImplementedError(self.set)
 
 
-class ConfigDict(Config):
+class ConfigDict(Config, DictMixin):
     """Git configuration stored in a dictionary."""
 
     def __init__(self, values=None):
         """Create a new ConfigDict."""
         if values is None:
-            values = {}
+            values = OrderedDict()
         self._values = values
 
     def __repr__(self):
@@ -90,6 +97,15 @@ class ConfigDict(Config):
             isinstance(other, self.__class__) and
             other._values == self._values)
 
+    def __getitem__(self, key):
+        return self._values[key]
+
+    def __setitem__(self, key, value):
+        self._values[key] = value
+
+    def keys(self):
+        return self._values.keys()
+
     @classmethod
     def _parse_setting(cls, name):
         parts = name.split(".")
@@ -111,7 +127,7 @@ class ConfigDict(Config):
     def set(self, section, name, value):
         if isinstance(section, basestring):
             section = (section, )
-        self._values.setdefault(section, {})[name] = value
+        self._values.setdefault(section, OrderedDict())[name] = value
 
 
 def _format_string(value):
@@ -225,7 +241,7 @@ class ConfigFile(ConfigDict):
                             section = (pts[0], pts[1])
                         else:
                             section = (pts[0], )
-                    ret._values[section] = {}
+                    ret._values[section] = OrderedDict()
                 if _strip_comments(line).strip() == "":
                     continue
                 if section is None:
@@ -293,7 +309,7 @@ class ConfigFile(ConfigDict):
             else:
                 f.write("[%s \"%s\"]\n" % (section_name, subsection_name))
             for key, value in values.iteritems():
-                f.write("%s = %s\n" % (key, _escape_value(value)))
+                f.write("\t%s = %s\n" % (key, _escape_value(value)))
 
 
 class StackedConfig(Config):
