@@ -51,7 +51,7 @@ _PARENT_HEADER = "parent"
 _AUTHOR_HEADER = "author"
 _COMMITTER_HEADER = "committer"
 _ENCODING_HEADER = "encoding"
-
+_MERGETAG_HEADER = "mergetag"
 
 # Header fields for objects
 _OBJECT_HEADER = "object"
@@ -583,12 +583,18 @@ def _parse_tag_or_commit(text):
         field named None for the freeform tag/commit text.
     """
     f = StringIO(text)
+    k = None
+    v = ""
     for l in f:
-        l = l.rstrip("\n")
-        if l == "":
-            # Empty line indicates end of headers
-            break
-        yield l.split(" ", 1)
+        if l.startswith(" "):
+            v += l[1:]
+        else:
+            if k is not None:
+                yield (k, v.rstrip("\n"))
+            if l == "\n":
+                # Empty line indicates end of headers
+                break
+            (k, v) = l.split(" ", 1)
     yield (None, f.read())
     f.close()
 
@@ -1038,12 +1044,13 @@ class Commit(ShaFile):
                  '_commit_timezone_neg_utc', '_commit_time',
                  '_author_time', '_author_timezone', '_commit_timezone',
                  '_author', '_committer', '_parents', '_extra',
-                 '_encoding', '_tree', '_message')
+                 '_encoding', '_tree', '_message', '_mergetag')
 
     def __init__(self):
         super(Commit, self).__init__()
         self._parents = []
         self._encoding = None
+        self._mergetag = []
         self._extra = []
         self._author_timezone_neg_utc = False
         self._commit_timezone_neg_utc = False
@@ -1059,7 +1066,7 @@ class Commit(ShaFile):
         self._parents = []
         self._extra = []
         self._author = None
-        for field, value in parse_commit(''.join(self._chunked_text)):
+        for field, value in parse_commit(''.join(chunks)):
             if field == _TREE_HEADER:
                 self._tree = value
             elif field == _PARENT_HEADER:
@@ -1078,6 +1085,8 @@ class Commit(ShaFile):
                 self._encoding = value
             elif field is None:
                 self._message = value
+            elif field == _MERGETAG_HEADER:
+                self._mergetag.append(Tag.from_string(value + "\n"))
             else:
                 self._extra.append((field, value))
 
@@ -1132,6 +1141,16 @@ class Commit(ShaFile):
                           self._commit_timezone_neg_utc)))
         if self.encoding:
             chunks.append("%s %s\n" % (_ENCODING_HEADER, self.encoding))
+        for mergetag in self.mergetag:
+            mergetag_chunks = mergetag.as_raw_string().split("\n")
+
+            chunks.append("%s %s\n" % (_MERGETAG_HEADER, mergetag_chunks[0]))
+            # Embedded extra header needs leading space
+            for chunk in mergetag_chunks[1:]:
+                chunks.append(" %s\n" % chunk)
+
+            # No trailing empty line
+            chunks[-1] = chunks[-1].rstrip(" \n")
         for k, v in self.extra:
             if "\n" in k or "\n" in v:
                 raise AssertionError("newline in extra data: %r -> %r" % (k, v))
@@ -1186,6 +1205,9 @@ class Commit(ShaFile):
     encoding = serializable_property("encoding",
         "Encoding of the commit message.")
 
+    mergetag = serializable_property("mergetag",
+        "Associated signed tag.")
+
 
 OBJECT_CLASSES = (
     Commit,
