@@ -117,10 +117,12 @@ def self_test_suite():
         'diff_tree',
         'fastexport',
         'file',
+        'hooks',
         'index',
         'lru_cache',
         'objects',
         'object_store',
+        'missing_obj_finder',
         'pack',
         'patch',
         'protocol',
@@ -161,6 +163,13 @@ def nocompat_test_suite():
     return result
 
 
+def compat_test_suite():
+    result = unittest.TestSuite()
+    from dulwich.tests.compat import test_suite as compat_test_suite
+    result.addTests(compat_test_suite())
+    return result
+
+
 def test_suite():
     result = unittest.TestSuite()
     result.addTests(self_test_suite())
